module TicketAbstractorClient
  module Base
    class Attachment
      attr_reader :file_path, :data_hash, :external_created_at, :communications_stack

      # Carrierwave uses the same regexp
      SANITIZE_REGEXP = /[^[:word:]\.\-\+]/

      class << self
        def fetch
          method_not_implemented __method__
        end

        protected

        def method_not_implemented(method_name)
          raise Errors::NotImplementedError, "#{self}##{method_name} is not implemented"
        end

        def save_content(content, original_filename)
          tmp_file = Tempfile.new(Time.now.to_s)
          tmp_file.binmode
          tmp_file.write(content)
          tmp_file.close
          set_original_name(tmp_file.path, original_filename)
        end

        def set_original_name(filepath, original_name)
          sanitized_name = original_name.gsub(SANITIZE_REGEXP, "_")
          tmp_basename = File.basename(filepath)
          tmp_path = filepath.gsub(tmp_basename, '')
          FileUtils.mv(filepath, File.join(tmp_path, sanitized_name))
          File.join(tmp_path, sanitized_name)
        end
      end

      def initialize(opts)
        @file_path = opts[:file_path]
        @external_created_at = opts[:external_created_at]
        @data_hash = opts[:data_hash]
        @communications_stack = opts[:communications_stack] || []
        set_data_hash! if @data_hash.blank?
      end

      def sync!
        self.class.method_not_implemented __method__
      end

      def set_data_hash!
        return if self.file_path.blank?
        file = File.new(self.file_path, 'rb')
        @data_hash = Digest::SHA512.hexdigest(file.read)
        file.close
      end

      def to_hash
        { file_path: @file_path, external_created_at: @external_created_at, data_hash: @data_hash }
      end

      def to_json
        self.to_hash.to_json
      end
    end
  end
end
