module TicketAbstractorClient
  module Base
    class Comment
      attr_reader :author, :body, :data_hash, :external_created_at, :communications_stack

      def self.fetch(*opts)
        method_not_implemented __method__
      end

      def initialize(opts)
        opts = opts.with_indifferent_access
        @author = opts[:author]
        @body = opts.fetch(:body, nil) || raise(Errors::CommentArgumentError, 'Body is not given')
        @external_created_at = opts[:external_created_at]
        @data_hash = opts[:data_hash]
        @communications_stack = opts[:communications_stack] || []
        set_data_hash! if @data_hash.blank?
      end

      def set_data_hash!
        content = "#{@external_created_at}:#{@author}:#{@body}"
        @data_hash = Digest::SHA512.hexdigest(content)
      end

      def sync!
        self.class.method_not_implemented __method__
      end

      def to_hash
        { author: @author, body: @body, external_created_at: @external_created_at, data_hash: @data_hash }
      end

      def to_json
        self.to_hash.to_json
      end

      protected

      def self.method_not_implemented(method_name)
        raise Errors::NotImplementedError, "#{self}##{method_name} is not implemented"
      end
    end
  end
end
