module TicketAbstractorClient
  module Base
    module ResponseHandler
      extend ActiveSupport::Concern

      def with_response_handling
        response = yield

        raise(error_by_context(response['context']), response['error'].to_s) if response.key?('error')

        response['result']
      end

      private

      def error_by_context(context)
        modulo, klass = context.split('::')
        error_class = "TicketAbstractorClient::#{modulo}::Errors::#{klass}".safe_constantize
        error_class.presence || unexpected_error_klass
      end

      def unexpected_error_klass
        modulo = self.class.name.deconstantize.split('::').last
        "TicketAbstractorClient::#{modulo}::Errors::UnexpectedError".safe_constantize
      end
    end
  end
end
