module TicketAbstractorClient
  module Base
    class Ticket
      attr_reader :attachments, :comments, :changes, :status
      attr_accessor :ticket_id, :endpoint, :project, :fields, :communications_stack

      def initialize(opts = {})
        opts = opts.with_indifferent_access
        @ticket_id = opts[:ticket_id]
        @fields = opts.fetch(:fields, {}).with_indifferent_access
        @endpoint = opts[:endpoint] || raise(Errors::TicketArgumentError, 'Endpoint is not given')
        @project = opts[:project]
        @communications_stack = opts[:communications_stack] || []
        @attachments ||= []
        @comments ||= []
        initialize_changes!
        mark_changes!
      end

      def add_attachment(attachment)
        self.class.not_implemented __method__
      end

      def add_comment(comment)
        self.class.not_implemented __method__
      end

      def any_changes?
        @changes.values.any? { |value| value.is_a?(Fixnum) ? !value.zero? : value.present? }
      end

      def reset_changes!
        initialize_changes!
      end

      def status=(status)
        return if @status == status
        @changes[:new_status] = true
        @status = status
      end

      def sync!
        self.class.not_implemented __method__
      end

      def to_hash
        {
          ticket_id: self.ticket_id,
          status: self.status,
          updated_at: self.updated_at,
          fields: self.fields,
          comments: self.comments.map(&:to_hash),
          attachments: self.attachments.map(&:to_hash)
        }
      end

      def to_json
        to_hash.to_json
      end

      def updated_at
        self.class.not_implemented __method__
      end

      protected

      def self.not_implemented(method_name)
        raise Errors::NotImplementedError, "#{self}##{__method__} is not implemented"
      end

      def initialize_changes!
        @changes = { create: false, update: false, new_status: false, new_attachments: 0, new_comments: 0 }
      end

      def mark_changes!
        return @changes[:create] = true if @ticket_id.blank?

        @changes[:update] = true if @fields.present?
      end
    end
  end
end
