module TicketAbstractorClient
  class Configuration
    attr_accessor :ticket_abstractor_url, :security_token, :snow_display_value, :jira_fields_meta_hash,
      :jira_meta_expiration_period, :ssl_options, :trace_communications

    DEFAULT_JIRA_META_EXPIRATION_PERIOD = 604800 # 1 week

    SNOW_DISPLAY_VALUES_LIST = [
      SNOW_DISPLAY_VALUE_ALL = :all,
      SNOW_DISPLAY_VALUE_TRUE = true,
      SNOW_DISPLAY_VALUE_FALSE = false
    ].freeze

    DEFAULT_SNOW_DISPLAY_VALUE = SNOW_DISPLAY_VALUE_FALSE

    DEFAULT_SSL_OPTIONS = { verify_ssl: true }.freeze

    DEFAULT_TRACE_COMMUNICATIONS = false

    def jira_tickets_filter_class
      @jira_tickets_filter_class.presence
    end

    def jira_tickets_filter_class=(filter_class)
      raise_configuration_error(filter_class) unless filter_class < Base::TicketsFilter

      @jira_tickets_filter_class = filter_class
    end

    def jira_fields_meta(endpoint = nil)
      @jira_fields_meta_hash ||= {}

      if endpoint.present?
        @jira_fields_meta_hash[endpoint.to_sym] ||= ClientHelper.get_jira_fields_meta(endpoint)[endpoint.to_sym]
      else
        @jira_fields_meta_hash ||= ClientHelper.get_jira_fields_meta
      end
    end

    def jira_meta_expiration_period
      @jira_meta_expiration_period ||= DEFAULT_JIRA_META_EXPIRATION_PERIOD
    end

    def trace_communications
      @trace_communications ||= DEFAULT_TRACE_COMMUNICATIONS
    end

    def snow_display_value
      @snow_display_value ||= DEFAULT_SNOW_DISPLAY_VALUE
    end

    def snow_tickets_filter_class
      @snow_tickets_filter_class.presence
    end

    def snow_tickets_filter_class=(filter_class)
      raise_configuration_error(filter_class) unless filter_class < Base::TicketsFilter

      @snow_tickets_filter_class = filter_class
    end

    def ssl_options
      @ssl_options ||= DEFAULT_SSL_OPTIONS
    end

    private

    def raise_configuration_error(filter_class)
      message = "Filter class '#{filter_class.to_s}' isn't a subclass of Base::TicketsFilter"
      raise Base::Errors::ConfigurationError, message
    end
  end
end
