module TicketAbstractorClient
  module Jira
    class Attachment < Base::Attachment
      attr_accessor :ticket_id, :endpoint, :project

      class << self
        def fetch_by_id_and_name(id, name, endpoint)
          file_path = download_attachment(id, name, endpoint)
          new(file_path: file_path, endpoint: endpoint)
        end

        def fetch_by_ticket_id(ticket_id, endpoint)
          ticket = Client.new(endpoint).get_ticket_by_id(ticket_id: ticket_id, fields: :attachment)

          ticket['fields']['attachment'].each_with_object([]) do |hash, attachments|
            attachment = fetch_by_id_and_name(hash['id'], hash['filename'], endpoint)
            attachment.ticket_id = ticket_id
            attachments << attachment
          end
        end
        alias_method :fetch, :fetch_by_ticket_id

        protected

        def download_attachment(id, name, endpoint)
          file_content = Client.new(endpoint).get_attachment(id: id, name: name)
          save_content(file_content, "#{id}_#{name}")
        end
      end

      def initialize(opts)
        super(opts)
        @ticket_id, @endpoint, @project = opts.values_at(:ticket_id, :endpoint, :project)
      end

      def sync!
        response = Client.new(@endpoint).create_attachment(self)
        @external_created_at = response.first['created']
      end
    end
  end
end
