module TicketAbstractorClient
  module Jira
    class Client < Base::Client
      include Jira::ParamsBuilder

      def initialize(endpoint = 'tp')
        super()
        @jira_endpoint = endpoint
      end

      def create_attachment(attachment)
        args, params = build_attachment_params(attachment)
        post(__method__, args, params)
      end

      def create_comment(comment)
        post(__method__, build_comment_params(comment))
      end

      def create_ticket(ticket)
        post(__method__, build_ticket_params(ticket))
      end

      def endpoints
        get(__method__)
      end

      def get_all_fields
        get(__method__)
      end

      def get_attachment(opts)
        get(__method__, opts)
      end

      def get_comments(opts)
        get(__method__, opts)
      end

      def get_fields_by_project(opts)
        get(__method__, opts)
      end

      def get_ticket_by_id(opts)
        get(__method__, opts)
      end

      def get_ticket_status(opts)
        get(__method__, build_status_params(opts))
      end

      def get_tickets_by_jql(opts)
        get(__method__, opts)
      end

      def get_tickets_statuses(opts)
        get(__method__, opts)
      end

      def update_ticket(ticket)
        post(__method__, build_ticket_params(ticket))
      end

      def transit_ticket(ticket)
        post(__method__, build_transitions_params(ticket))
      end

      def transitions_list(ticket_id)
        get(__method__, { ticket_id: ticket_id })
      end

      protected

      def get(path, args = {}, params = {})
        super("jira/#{path}", args, params.merge(jira_endpoint: @jira_endpoint))
      end

      def post(path, args = {}, params = {})
        super("jira/#{path}", args, params.merge(jira_endpoint: @jira_endpoint))
      end
    end
  end
end
