module TicketAbstractorClient
  module Jira
    class Comment < Base::Comment
      attr_accessor :ticket_id, :endpoint, :project, :author_display_name

      def self.fetch(ticket_id, endpoint)
        comments_list = Client.new(endpoint).get_comments(ticket_id: ticket_id)
        comments_list['comments'].map do |raw_comment|
          new(
            author: raw_comment['author']['name'],
            author_display_name: raw_comment['author']['displayName'],
            body: raw_comment['body'],
            external_created_at: raw_comment['created'],
            ticket_id: ticket_id,
            endpoint: endpoint
          )
        end
      end

      def initialize(opts)
        super(opts)
        @author_display_name = opts[:author_display_name]
        @ticket_id, @endpoint, @project = opts.values_at(:ticket_id, :endpoint, :project)
      end

      def sync!
        created_comment_hash = Client.new(@endpoint).create_comment(self)
        @author = created_comment_hash['author']['name']
        @author_display_name = created_comment_hash['author']['displayName']
        @external_created_at = created_comment_hash['created']
        self.set_data_hash!
      end
    end
  end
end
