module TicketAbstractorClient
  module Jira
    class FieldsMeta < Hash
      attr_accessor :endpoint
      attr_reader :updated_at

      def initialize(endpoint)
        super()
        @endpoint = endpoint
        update!
      end

      def update!
        get_meta_data
        update(@meta_data)
      end

      def expired?
        expiration_period = TicketAbstractorClient.configuration.jira_meta_expiration_period
        (Time.now.to_i - @updated_at) > expiration_period
      end

      private

      def get_meta_data
        raw_meta_data = Client.new(@endpoint).get_all_fields
        @meta_data = filter_meta_data(raw_meta_data)
        @updated_at = Time.now.to_i
      end

      def filter_meta_data(raw_meta_data)
        raw_meta_data.each_with_object({}) { |field, meta| meta[field['id']] = field['schema'] }
      end
    end
  end
end
