module TicketAbstractorClient
  module ServiceNow
    class Client < Base::Client
      include ServiceNow::ParamsBuilder

      SYS_ID_LENGTH = 32
      TICKET_NOT_FOUND = 'Ticket not found or incorrect ID given'.freeze

      def initialize(endpoint = 'default')
        super()
        @service_now_endpoint = endpoint
      end

      def get_all_tickets(opts)
        get_tickets_by_query(build_all_tickets_params(opts))
      end

      def get_attachments(opts)
        opts[:ticket_id] = get_ticket_sys_id(opts)
        get(__method__, opts)
      end

      def get_attachment_file(opts)
        get(__method__, opts)
      end

      def get_comments(opts)
        opts[:table_name] = opts[:project]
        opts[:sys_id] = get_ticket_sys_id(opts)
        opts[:display_value] = TicketAbstractorClient.configuration.snow_display_value if opts[:display_value].nil?
        get(__method__, opts)
      end

      def endpoints
        get(__method__)
      end

      def get_ticket_by_id(opts)
        opts[:table_name] = opts[:project]
        opts[:sys_id] = get_ticket_sys_id(opts)
        opts[:display_value] = TicketAbstractorClient.configuration.snow_display_value if opts[:display_value].nil?
        get(__method__, opts)
      end

      def get_tickets_by_query(opts)
        opts[:table_name] = opts[:project]
        opts[:display_value] = TicketAbstractorClient.configuration.snow_display_value if opts[:display_value].nil?
        get(__method__, opts)
      end

      def get_users(opts)
        get(__method__, opts)
      end

      def create_attachment(attachment)
        attachment.ticket_id = get_ticket_sys_id(ticket_id: attachment.ticket_id, table_name: attachment.project)

        args, params = build_attachment_params(attachment)
        post(__method__, args, params)
      end

      def create_comment(comment)
        comment.ticket_id = get_ticket_sys_id(ticket_id: comment.ticket_id, table_name: comment.project)
        post(__method__, build_comment_params(comment))
      end

      def create_ticket(ticket)
        post(__method__, build_ticket_params(ticket))
      end

      def update_ticket(ticket)
        ticket.sys_id = get_ticket_sys_id(ticket_id: ticket.ticket_id, table_name: ticket.project)
        post(__method__, build_ticket_params(ticket))
      end

      def update_ticket_status(ticket)
        ticket.sys_id = get_ticket_sys_id(ticket_id: ticket.ticket.id, table_name: ticket.project)
        post(:update_ticket, build_status_params(ticket))
      end

      protected

      def get_ticket_sys_id(opts)
        return opts[:ticket_id] if opts[:ticket_id].length == SYS_ID_LENGTH

        response = get(__method__, opts)

        raise(Errors::NotFoundError, TICKET_NOT_FOUND) if response.blank?

        response[0]['sys_id']
      end

      def get(path, args = {}, params = {})
        super("service_now/#{path}", args, params.merge(service_now_endpoint: @service_now_endpoint))
      end

      def post(path, args = {}, params = {})
        super("service_now/#{path}", args, params.merge(service_now_endpoint: @service_now_endpoint))
      end
    end
  end
end
